from cryptography.fernet import Fernet, InvalidToken
from pathlib import Path
import base64


class SymmetricEncryption:
    """
    对称加密工具类,基于 Fernet (AES-128-CBC + HMAC-SHA256),安全且易于使用.

    支持从密钥文件加载密钥,用于加密和解密字符串数据.
    密钥必须是 32 字节的 URL-safe base64 编码字符串(Fernet 格式).
    """

    def __init__(
        self,
        key_path: str | Path | None = None,
    ) -> None:
        self.key_path: Path | None = None
        self.cipher: Fernet | None = None
        self.key_data: str | None = None

        # -------------------------------
        # 处理 key_path：转换为 Path
        # -------------------------------
        if key_path is None:
            pass  # 保持为 None
        elif isinstance(key_path, str) or isinstance(key_path, Path):
            self.key_path = Path(key_path)
        else:
            raise TypeError(
                f"key_path must be str, Path, or None, got {type(key_path)}"
            )

        # -------------------------------
        # 校验密钥文件路径（如果提供了）
        # -------------------------------
        if self.key_path is not None:
            self._validate_path(self.key_path, "Symmetric key file")
            self._load_key()

    def _validate_path(self, path: Path, name: str) -> None:
        """私有方法：验证路径存在且为文件"""
        if not path.exists():
            raise FileNotFoundError(f"{name} not found: {path}")
        if not path.is_file():
            raise ValueError(f"{name} is not a file: {path}")

    def _load_key(self) -> None:
        """私有方法：从文件加载对称密钥并初始化 Fernet"""
        assert self.key_path is not None
        try:
            self.key_data = self.key_path.read_text().strip()
            self.cipher = Fernet(self.key_data)
        except Exception as e:
            raise ValueError(
                f"Failed to load or parse symmetric key from {self.key_path}: {e}"
            ) from e

    def encrypt(self, message: str) -> str:
        """使用对称密钥加密字符串，返回 Base64 编码的密文"""
        if self.cipher is None:
            raise ValueError("Symmetric key not loaded. Cannot encrypt.")

        try:
            encrypted_data = self.cipher.encrypt(message.encode("utf-8"))
            return base64.b64encode(encrypted_data).decode("ascii")
        except Exception as e:
            raise ValueError(f"Encryption failed: {e}") from e

    def decrypt(self, ciphertext_base64: str) -> str:
        """解密 Base64 编码的密文，返回原始字符串"""
        if self.cipher is None:
            raise ValueError("Symmetric key not loaded. Cannot decrypt.")

        try:
            ciphertext = base64.b64decode(ciphertext_base64)
            decrypted_data = self.cipher.decrypt(ciphertext)
            return decrypted_data.decode("utf-8")
        except InvalidToken:
            raise ValueError(
                "Decryption failed: invalid token (possibly corrupted or wrong key)"
            )
        except Exception as e:
            raise ValueError(f"Decryption failed: {e}") from e

    def generate_key(self, set_this: bool = False) -> str:
        """生成一个新的安全随机密钥(Base64 编码字符串)"""
        key = Fernet.generate_key().decode("ascii")  # this is b64 bytes
        if set_this:
            self.key_data = key
            self.cipher = Fernet(key)
        return key

    def save_key(self, path: str | Path) -> None:
        """将密钥保存到文件"""
        key_path = Path(path)
        key_path.write_text(self.key_data)



import platform
import os
import subprocess
import winreg
from typing import Optional
import ctypes
from ctypes import wintypes


class HardwareData:
    def __init__(self):
        """初始化硬件数据对象，自动获取所有硬件信息"""
        self.system_volume_serial: Optional[str] = self.get_system_volume_serial()
        self.system_volume_name: Optional[str] = self.get_system_volume_name()
        self.computer_name: Optional[str] = self.get_computer_name()
        self.cpu_type: Optional[str] = self.get_cpu_type()
        self.bios_info: Optional[str] = self.get_bios_info()
        self.windows_serial: Optional[str] = self.get_windows_serial()
        self.disk_serial: Optional[str] = self.get_disk_serial()
        self.windows_username: Optional[str] = self.get_windows_username()

    @staticmethod
    def get_system_volume_serial() -> Optional[str]:
        """获取系统卷标序列号（使用Windows API）"""
        try:
            # 定义函数原型
            kernel32 = ctypes.windll.kernel32
            GetVolumeInformationW = kernel32.GetVolumeInformationW
            GetVolumeInformationW.argtypes = [
                wintypes.LPCWSTR,  # lpRootPathName
                wintypes.LPWSTR,  # lpVolumeNameBuffer
                wintypes.DWORD,  # nVolumeNameSize
                ctypes.POINTER(wintypes.DWORD),  # lpVolumeSerialNumber
                ctypes.POINTER(wintypes.DWORD),  # lpMaximumComponentLength
                ctypes.POINTER(wintypes.DWORD),  # lpFileSystemFlags
                wintypes.LPWSTR,  # lpFileSystemNameBuffer
                wintypes.DWORD,  # nFileSystemNameSize
            ]
            GetVolumeInformationW.restype = wintypes.BOOL

            # 准备缓冲区
            volume_name_buffer = ctypes.create_unicode_buffer(256)
            file_system_name_buffer = ctypes.create_unicode_buffer(256)
            volume_serial_number = wintypes.DWORD()
            max_component_length = wintypes.DWORD()
            file_system_flags = wintypes.DWORD()

            # 调用 API
            result = GetVolumeInformationW(
                "C:\\",
                volume_name_buffer,
                256,
                ctypes.byref(volume_serial_number),
                ctypes.byref(max_component_length),
                ctypes.byref(file_system_flags),
                file_system_name_buffer,
                256,
            )

            if result:
                # 格式化为十六进制字符串
                serial = volume_serial_number.value
                return f"{serial:08X}"
            return None
        except Exception:
            return None

    @staticmethod
    def get_system_volume_name() -> Optional[str]:
        """获取系统卷标名称（使用Windows API）"""
        try:
            kernel32 = ctypes.windll.kernel32
            GetVolumeInformationW = kernel32.GetVolumeInformationW
            GetVolumeInformationW.argtypes = [
                wintypes.LPCWSTR,  # lpRootPathName
                wintypes.LPWSTR,  # lpVolumeNameBuffer
                wintypes.DWORD,  # nVolumeNameSize
                ctypes.POINTER(wintypes.DWORD),  # lpVolumeSerialNumber
                ctypes.POINTER(wintypes.DWORD),  # lpMaximumComponentLength
                ctypes.POINTER(wintypes.DWORD),  # lpFileSystemFlags
                wintypes.LPWSTR,  # lpFileSystemNameBuffer
                wintypes.DWORD,  # nFileSystemNameSize
            ]
            GetVolumeInformationW.restype = wintypes.BOOL

            volume_name_buffer = ctypes.create_unicode_buffer(256)
            file_system_name_buffer = ctypes.create_unicode_buffer(256)
            volume_serial_number = wintypes.DWORD()
            max_component_length = wintypes.DWORD()
            file_system_flags = wintypes.DWORD()

            result = GetVolumeInformationW(
                "C:\\",
                volume_name_buffer,
                256,
                ctypes.byref(volume_serial_number),
                ctypes.byref(max_component_length),
                ctypes.byref(file_system_flags),
                file_system_name_buffer,
                256,
            )

            if result and volume_name_buffer.value:
                return volume_name_buffer.value
            return None
        except Exception:
            return None

    @staticmethod
    def get_computer_name() -> Optional[str]:
        """获取计算机名称（使用环境变量）"""
        try:
            return os.environ.get("COMPUTERNAME", platform.node())
        except Exception:
            return platform.node()

    @staticmethod
    def get_cpu_type() -> Optional[str]:
        """获取CPU类型（使用注册表）"""
        try:
            # 从注册表获取CPU信息
            key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                r"HARDWARE\DESCRIPTION\System\CentralProcessor\0",
            )
            cpu_name = winreg.QueryValueEx(key, "ProcessorNameString")[0]
            winreg.CloseKey(key)
            return cpu_name.strip()
        except Exception:
            return platform.processor()

    @staticmethod
    def get_bios_info() -> Optional[str]:
        """获取主板BIOS信息（使用注册表）"""
        try:
            # 获取BIOS版本
            key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE, r"HARDWARE\DESCRIPTION\System\BIOS"
            )
            bios_version = winreg.QueryValueEx(key, "BIOSVersion")[0]
            winreg.CloseKey(key)

            if isinstance(bios_version, list):
                return " ".join(bios_version)
            return str(bios_version)
        except Exception:
            return None

    @staticmethod
    def get_windows_serial() -> str:
        """
        Windows Product Serial Number
        """
        try:
            key_path = r"SOFTWARE\Microsoft\Windows NT\CurrentVersion"

            # Open registry key, force 64-bit view
            key = winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                key_path,
                0,
                winreg.KEY_READ | winreg.KEY_WOW64_64KEY,
            )

            try:
                product_id, _ = winreg.QueryValueEx(key, "ProductId")
            finally:
                winreg.CloseKey(key)

            # Validate result — 如果无效，直接抛 ValueError，不要被外层 except 捕获
            if isinstance(product_id, str) and product_id.strip():
                return product_id.strip()
            else:
                raise ValueError("ProductId is empty or invalid")

        except FileNotFoundError:
            raise FileNotFoundError(
                "Registry path not found. Ensure you're running on Windows "
                "and the key exists under HKLM\\SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion"
            )
        except PermissionError:
            raise PermissionError(
                "Insufficient permissions to read registry. Try running as Administrator."
            )
        except OSError as e:
            raise OSError(f"OS-level registry error: {e}")

    @staticmethod
    def get_disk_serial() -> Optional[str]:
        """获取硬盘序列号（使用PowerShell）"""
        try:
            # 主方法
            cmd = [
                "powershell",
                "-Command",
                "Get-PhysicalDisk | Select-Object -First 1 SerialNumber | Format-List",
            ]
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=15)
            if result.returncode == 0:
                output = result.stdout.strip()
                lines = output.split("\n")
                for line in lines:
                    if "SerialNumber" in line:
                        serial = line.split(":", 1)[1].strip()
                        if serial:
                            return serial
        except Exception:
            pass  # 忽略主方法异常

        # 备用方法
        try:
            cmd = [
                "powershell",
                "-Command",
                "(Get-WmiObject -Class Win32_DiskDrive | Select-Object -First 1).SerialNumber",
            ]
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=15)
            if result.returncode == 0:
                serial = result.stdout.strip()
                if serial:
                    return serial
        except Exception:
            pass

        return None

    @staticmethod
    def get_windows_username() -> Optional[str]:
        """获取Windows用户名（使用环境变量）"""
        try:
            return os.environ.get("USERNAME", os.environ.get("USER", None))
        except Exception:
            return None

    def __str__(self) -> str:
        """返回硬件信息的字符串表示"""
        return f"""Hardware Data:
- System Volume Serial: {self.system_volume_serial}
- System Volume Name: {self.system_volume_name}
- Computer Name: {self.computer_name}
- CPU Type: {self.cpu_type}
- BIOS Info: {self.bios_info}
- Windows Serial: {self.windows_serial}
- Disk Serial: {self.disk_serial}
- Windows Username: {self.windows_username}"""

    def to_dict(self) -> dict:
        """返回硬件信息的字典表示"""
        return {
            "system_volume_serial": self.system_volume_serial,
            "system_volume_name": self.system_volume_name,
            "computer_name": self.computer_name,
            "cpu_type": self.cpu_type,
            "bios_info": self.bios_info,
            "windows_serial": self.windows_serial,
            "disk_serial": self.disk_serial,
            "windows_username": self.windows_username,
        }



from cryptography.hazmat.primitives.asymmetric import padding, rsa
from cryptography.hazmat.primitives import serialization, hashes
from pathlib import Path
import base64


class AsymmetricEncryption:
    def __init__(
        self,
        public_key_path: str | Path | None = None,
        private_key_path: str | Path | None = None,
    ) -> None:

        # 转换为 Path 并验证（如果提供了路径）
        if public_key_path is None:
            pass  # 保持为 None
        elif isinstance(public_key_path, str) or isinstance(public_key_path, Path):
            public_key_path = Path(public_key_path)
        else:
            raise TypeError(
                f"public_key_path must be str, Path, or None, got {type(public_key_path)}"
            )

        if private_key_path is None:
            pass  # 保持为 None
        elif isinstance(private_key_path, str) or isinstance(private_key_path, Path):
            private_key_path = Path(private_key_path)
        else:
            raise TypeError(
                f"private_key_path must be str, Path, or None, got {type(private_key_path)}"
            )

        # 校验公钥路径
        if public_key_path is not None:
            self._validate_path(public_key_path, "Public key file")
        # 校验私钥路径
        if private_key_path is not None:
            self._validate_path(private_key_path, "Private key file")

        # 所有校验通过，安全赋值
        self.public_key_path: Path | None = public_key_path
        self.private_key_path: Path | None = private_key_path

        self.public_key: rsa.RSAPublicKey | None = None
        self.private_key: rsa.RSAPrivateKey | None = None

        # 安全加载（此时路径已验证存在）
        if self.public_key_path:
            self._load_public_key()
        if self.private_key_path:
            self._load_private_key()

    def _load_public_key(self) -> None:
        """私有方法：加载公钥"""
        assert self.public_key_path is not None
        try:
            key_data = self.public_key_path.read_text().strip()
            # 检查是否是有效的PEM格式
            if not key_data.startswith("-----BEGIN"):
                raise ValueError("Invalid PEM format: missing BEGIN header")
            self.public_key = serialization.load_pem_public_key(
                key_data.encode("utf-8")
            )
        except ValueError as e:
            if "Unable to load PEM file" in str(e) or "MalformedFraming" in str(e):
                raise ValueError(
                    f"Failed to load public key from {self.public_key_path}: "
                    f"Invalid or corrupted PEM file format"
                ) from e
            else:
                raise ValueError(
                    f"Failed to load public key from {self.public_key_path}: {e}"
                ) from e
        except Exception as e:
            raise ValueError(
                f"Failed to load public key from {self.public_key_path}: {e}"
            ) from e

    def _load_private_key(self) -> None:
        """私有方法：加载私钥"""
        assert self.private_key_path is not None
        try:
            key_data = self.private_key_path.read_text().strip()
            # 检查是否是有效的PEM格式
            if not key_data.startswith("-----BEGIN"):
                raise ValueError("Invalid PEM format: missing BEGIN header")
            self.private_key = serialization.load_pem_private_key(
                key_data.encode("utf-8"), None
            )
        except ValueError as e:
            if "Unable to load PEM file" in str(e) or "MalformedFraming" in str(e):
                raise ValueError(
                    f"Failed to load private key from {self.private_key_path}: "
                    f"Invalid or corrupted PEM file format"
                ) from e
            else:
                raise ValueError(
                    f"Failed to load private key from {self.private_key_path}: {e}"
                ) from e
        except Exception as e:
            raise ValueError(
                f"Failed to load private key from {self.private_key_path}: {e}"
            ) from e

    def encrypt(self, message):
        if self.public_key is None:
            raise ValueError("Public key not loaded. Use load_public_key() to load it.")

        # 使用公钥加密
        ciphertext = self.public_key.encrypt(
            message.encode("utf-8"),
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None,
            ),
        )
        return base64.b64encode(ciphertext).decode("ascii")

    def decrypt(self, ciphertext_base64):
        if self.private_key is None:
            raise ValueError(
                "Private key not loaded. Use load_private_key() to load it."
            )

        # Base64 解码并解密
        ciphertext = base64.b64decode(ciphertext_base64)
        decrypted_message = self.private_key.decrypt(
            ciphertext,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None,
            ),
        )
        return decrypted_message.decode("utf-8")

    def generate_keys(
        self, key_size: int = 2048, set_this: bool = False
    ) -> tuple[rsa.RSAPrivateKey, rsa.RSAPublicKey]:
        """
        生成一个新的 RSA 密钥对。

        :param key_size: 密钥长度，默认为 2048 位
        """
        if key_size < 512:
            raise ValueError("Key size must be at least 512 bits.")

        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=key_size,
        )
        public_key = private_key.public_key()

        if set_this:
            self.private_key = private_key
            self.public_key = public_key

        return private_key, public_key

    def save_keys(
        self, public_key_path: str | Path, private_key_path: str | Path
    ) -> None:
        """
        将当前已生成的公钥和私钥保存为 PEM 文件。

        :param public_key_path: 公钥保存路径
        :param private_key_path: 私钥保存路径
        """
        if self.public_key is None or self.private_key is None:
            raise ValueError(
                "Keys have not been generated yet. Call generate_keys() first."
            )

        if isinstance(public_key_path, str):
            public_path = Path(public_key_path)
        else:
            public_path = public_key_path

        if isinstance(private_key_path, str):
            private_path = Path(private_key_path)
        else:
            private_path = private_key_path

        # 序列化并写入公钥
        public_pem = self.public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
        public_path.write_bytes(public_pem)

        # 序列化并写入私钥
        private_pem = self.private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )
        private_path.write_bytes(private_pem)

        # 更新实例变量
        self.public_key_path = public_path
        self.private_key_path = private_path

    def _validate_path(self, path: Path, name: str) -> None:
        if not path.exists():
            raise FileNotFoundError(f"{name} not found: {path}")
        if not path.is_file():
            raise ValueError(f"{name} is not a file: {path}")
